!*****************************************************************************
!                                                                            *
!  RAINBOW.F90                                                               *
!                                                                            *
!  This program demonstrates color palette cycling.                          *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGrainbow'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = NULL
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Color Cycling'C,        &  ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

type (T_PAINTSTRUCT) ps
integer*4 cxClient, cyClient
integer*4 hdc, hpal, hvb
integer*4 start
integer*4 color, xlen, ylen
integer*4 iret

integer*1 RGBvalues
common    RGBvalues(0:2*24*3-1)         ! two sets of 24 RGB triplets

save cxClient, cyClient, hdc, hpal, hvb, start

WindowProc = 0

select case (message)

   case (WM_CREATE)
      ! create the logical palette
      hdc = GetDC(window)
      call fg_setdc(hdc)
      call fill_color_palette
      hpal = fg_logpal(10,24,RGBvalues)
      call fg_realize(hpal)

      ! create a 640x480 virtual buffer
      call fg_vbinit
      hvb = fg_vballoc(640,480)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      ! construct a crude image of a rainbow
      call fg_setcolor(255)
      call fg_fillpage
      call fg_setclip(0,639,0,300)
      call fg_move(320,300)
      xlen = 240
      ylen = 120
      do color = 10,33
         call fg_setcolor(color)
         call fg_ellipsef(xlen,ylen)
         xlen = xlen - 4
         ylen = ylen - 3
      end do
      call fg_setcolor(255)
      call fg_ellipsef(xlen,ylen)
      call fg_setclip(0,639,0,479)

      ! starting index into the array of color values
      start = 0

      ! start the 50ms timer
      iret = SetTimer(window,1,50,0,0)
      return

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_TIMER)
      if (GetActiveWindow() == window) then
         start = modulo(start+3,72)
         call fg_setdacs(10,24,RGBvalues(start))
         if (fg_colors() > 8) then
            call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
         end if
      end if
      return

   case (WM_DESTROY)
      iret = KillTimer(window,1)
      call fg_vbclose
      call fg_vbfree(hvb)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  fill_color_palette                                                        *
!                                                                            *
!  Set up the colors for the application's logical palette in the RGBvalues  *
!  array. The logical palette will contain 24 non-system colors (indices 10  *
!  to 33) defining the initial RGB values for the colors being cycled.       *
!                                                                            *
!  Note that we store two identical sets of 24 RGB triplets in RGBvalues. We *
!  can then perform color cycling without having to worry about wrapping to  *
!  the start of the array because the index pointing to the starting RGB     *
!  triplet never extends beyond the first set of 24 RGB triplets.            *
!                                                                            *
!*****************************************************************************

subroutine fill_color_palette

integer*4 i

integer*1 RGBvalues
common    RGBvalues(0:2*24*3-1)         ! two sets of 24 RGB triplets

integer*1 colors(0:24*3-1)
data colors / &
   182,182,255, 198,182,255, 218,182,255, 234,182,255, 255,182,255, &
   255,182,234, 255,182,218, 255,182,198, 255,182,182, 255,198,182, &
   255,218,182, 255,234,182, 255,255,182, 234,255,182, 218,255,182, &
   198,255,182, 182,255,182, 182,255,198, 182,255,218, 182,255,234, &
   182,255,255, 182,234,255, 182,218,255, 182,198,255/

! set up two identical sets of the 24 colors in the RGBvalues array
do i = 0,24*3-1
   RGBvalues(i) = colors(i)
   RGBvalues(24*3+i) = colors(i)
end do

return
end
